
/*
** BBBSPWD.C - Update password in BBBS user database
*/

/*
** Copyright (C) 2003, Bob Swift. All rights reserved.
**
** The copyright holder makes no representations concerning either
** the merchantability of this software or the suitability of this
** software for any particular purpose. It is provided "as is"
** without express or implied warranty of any kind.
**
** These notices must be retained in any copies of any part of this
** documentation and/or software.
**
** The program incorporates the source code for the "RSA Data Security, Inc.
** MD5 Message-Digest Algorithm" as found in RFC 1321.  That code is copyrighted:
**
**	Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991.
**	All rights reserved.
**
** See the source code for copyright restrictions.
*/

#define TITLE   "BBBSPWD"       /*  Program Name    */
#define DATES   "2003"          /*  Copyright Date  */
#define VERSION "1.01"          /*  Version Number  */

/*
** Identify target operating system
*/
#ifdef OS2
#define OPSYSTEM "OS/2"
#endif
#ifdef DOS
#define OPSYSTEM "DOS"
#endif
#ifdef WIN
#define OPSYSTEM "Windows"
#endif
#ifdef LINUX
#define OPSYSTEM "Linux"
#pragma pack(1)
#endif

/*
** Standard includes
*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef LINUX
#include <unistd.h>
#else
#include <mem.h>
#endif

/*
** Program specific includes
*/
#include "global.h"
#include "md5.h"
#include "bbbsdef.h"

/*
** Length of test block, number of test blocks.
*/
#define TEST_BLOCK_LEN 1000
#define TEST_BLOCK_COUNT 1000

/*
** Function prototypes
*/
int min(int, int);
void strupper(char *, char *);
void ShowError(int);
static void MDString(unsigned char[16], char *);

/*
** Main program entry
*/
int main(argc, argv)
int argc;
char *argv[];
{
  int i, j, quiet;
  char upswd[64];
  char uname[64];
  char ufile[255];
  long recsize;
  unsigned recnum, maxrecs;
  unsigned char digest[16];
  FILE *userfile;
  struct userrec caller;

  strcpy(upswd, "");
  strcpy(uname, "");
  strcpy(ufile, "bbbsuser.dat");

  j = argc - 1;
  quiet = 0;

/*
  printf("argc = %u\n",argc);
  for (i = 1; i < argc; i++) {
    printf("argv[%u] = \"%s\"\n",i,argv[i]);
  }
*/
  if (argc < 2)
    ShowError(-1);
  for (i = 1; i < argc; i++) {
    if (argv[i][0] == '-' && tolower(argv[i][1]) == 'f') {
      i++;
      i = min(i, j);
      strcpy(ufile, argv[i]);
    }
    else if (argv[i][0] == '-' && tolower(argv[i][1]) == 'p') {
      i++;
      i = min(i, j);
      strupper(upswd, argv[i]);
    }
    else if (argv[i][0] == '-' && tolower(argv[i][1]) == 'u') {
      i++;
      i = min(i, j);
      strcpy(uname, argv[i]);
    }
    else if (argv[i][0] == '-' && tolower(argv[i][1]) == 'q') {
      quiet = 1;
    }
    else
      ShowError(-1);
  }
  if (upswd[0] == '\0')
    ShowError(1);
  if (uname[0] == '\0')
    ShowError(2);
  if (quiet == 0)
    printf("\nAssigning %s the password \"%s\"\n", uname, upswd);
  strupper(uname, uname);
  MDString(digest, upswd);
  recsize = (long) sizeof caller;
  userfile = fopen(ufile, "rb");
  if (userfile == NULL)
    ShowError(3);
  fseek(userfile, 0L, SEEK_END);
  maxrecs = (unsigned int) (ftell(userfile) / recsize);
  fclose(userfile);
  if (quiet == 0)
    printf("Checking %u user records.\n", maxrecs);
  userfile = fopen(ufile, "r+b");
  if (userfile == NULL)
    ShowError(3);
  j = 0;
  recnum = 0;
  while (recnum < maxrecs) {
    fseek(userfile, ((long) recnum) * recsize, 0);
    if (fread(&caller, recsize, 1, userfile) != 1) {
      fclose(userfile);
      ShowError(3);
    }
    if (strcmp(uname, caller.name) == 0) {
      if (quiet == 0)
        printf("Updating user record %u.\n", recnum + 1);
      i = 0;
      while (i < 16) {
        caller.password[i] = digest[i];
        i++;
      }
      fseek(userfile, ((long) recnum) * recsize, SEEK_SET);
      if (fwrite(&caller, recsize, 1, userfile) != 1)
        ShowError(5);
      recnum = maxrecs;
      j = 1;
    }
    recnum++;
  }
  fclose(userfile);
  if (j == 0)
    ShowError(4);
  if (quiet == 0)
    printf("User file update successful.\n\n");
  return (0);
}

/*
** Digests a string.
*/
static void MDString(digest, string)
unsigned char digest[16];
char *string;
{
  MD5_CTX context;
  unsigned int len = strlen(string);

  MD5Init(&context);
  MD5Update(&context, string, len);
  MD5Final(digest, &context);
}

/*
** Returns the lower input value.
*/
int min(one, two)
int one, two;
{
  int three;

  three = one;
  if (two < one)
    three = two;
  return (three);
}

/*
** Converts string to upper case.
*/
void strupper(strreturn, strinput)
char *strinput, *strreturn;
{
  for (; *strinput != '\0'; ++strinput, ++strreturn)
    *strreturn = toupper(*strinput);
  *strreturn = '\0';
}

/*
** Prints error and help screen.
*/
void ShowError(ernum)
int ernum;
{
  printf("\n");
  switch (ernum) {

    case 1:
      printf("**  Bad or missing password  **\n\n");
      break;

    case 2:
      printf("**  Bad or missing user name  **\n\n");
      break;

    case 3:
      printf("**  Unable to read user file  **\n\n");
      break;

    case 4:
      printf("**  Unable to find user record  **\n\n");
      break;

    case 5:
      printf("**  Unable to update user file  **\n\n");
      break;

    default:
      printf("%s (%s version %s)\n", TITLE, OPSYSTEM, VERSION);
      printf("Copyright %s, Bob Swift.  All rights reserved.\n\n", DATES);
      printf("This program allows you to update a BBBS user's ");
      printf("password from the\ncommand line.\n\n");
      printf("Usage:  bbbspwd [-q[uiet]] [-f \"user file\"] -p ");
      printf("\"password\" -u \"user name\"\n\nIf any of the ");
      printf("arguments (user file, password, or user name) ");
      printf("include spaces,\nthey must be enclosed within ");
      printf("quotes.  If the user file is not specified, the");
      printf("\nprogram will attempt to use bbbsuser.dat ");
      printf("in the current directory.  Both the\nuser name and ");
      printf("password must be specified.  Neither is case sensitive.\n\n");
      printf("Errorlevels returned:    0 - Password updated successfully.\n");
      printf("                         1 - Bad or missing password.\n");
      printf("                         2 - Bad or missing user name.\n");
      printf("                         3 - Unable to read user file.\n");
      printf("                         4 - Unable to find user record.\n");
      printf("                         5 - Unable to update user file.\n");
      printf("                        -1 - Bad or missing argument. ");
      printf("(This help screen.)\n\n");
  }
  exit(ernum);
}
